SystemOrganization addCategory: #GODEmail!

Object subclass: #MAIL
	instanceVariableNames: 'hostname password user port'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODEmail'!

!MAIL class methodsFor: 'instance creation' stamp: 'LFSA 11/2/2014 01:14'!
connectTo: hostname identifiedAs: user password: password
	^ self new hostname: hostname; port: (self defaultPortNumber); user: user; password: password.! !

!MAIL class methodsFor: 'instance creation' stamp: 'LFSA 11/2/2014 01:13'!
connectTo: hostname port: port identifiedAs: user password: password
	^ self new hostname: hostname; port: port; user: user; password: password.
	! !

!MAIL class methodsFor: 'accessing' stamp: 'LFSA 11/2/2014 01:02'!
defaultPortNumber
	self subclassResponsibility
	! !

!MAIL methodsFor: 'accessing' stamp: 'gfg 10/3/2014 19:51'!
hostname
	^hostname! !

!MAIL methodsFor: 'accessing' stamp: 'gfg 10/3/2014 19:51'!
hostname: aString
	hostname := aString! !

!MAIL methodsFor: 'initialize-release' stamp: 'gfg 10/3/2014 19:51'!
initialize
	self initializeConnectionInfo.! !

!MAIL methodsFor: 'initialize-release' stamp: 'LFSA 11/2/2014 01:06'!
initializeConnectionInfo
	hostname := ''.
	user := ''.
	password := ''.! !

!MAIL methodsFor: 'accessing' stamp: 'gfg 10/3/2014 19:51'!
password: aString
	password := aString! !

!MAIL methodsFor: 'accessing' stamp: 'LFSA 11/2/2014 01:15'!
port
	^port.! !

!MAIL methodsFor: 'accessing' stamp: 'LFSA 11/2/2014 00:58'!
port: aNumber
	port := aNumber! !

!MAIL methodsFor: 'accessing' stamp: 'gfg 10/3/2014 19:51'!
user
	^user! !

!MAIL methodsFor: 'accessing' stamp: 'gfg 10/3/2014 19:51'!
user: aString
	user := aString! !

!MAIL methodsFor: 'private' stamp: 'gfg 10/3/2014 19:51'!
validateConnectionInfo
	hostname isEmpty
		ifTrue: [^Error signal: 'Hostname value is empty'].

	user isEmpty
		ifTrue: [^Error signal: 'User value is empty'].
	
	password isEmpty
		ifTrue: [^Error signal: 'Password value is empty'].! !

MAIL subclass: #MAILReceiver
	instanceVariableNames: 'pop3Client cur cache'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODEmail'!

!MAILReceiver class methodsFor: 'converting' stamp: 'gfg 10/19/2014 18:34'!
convertToGODData: email
	"converts the email to a GODData"

	| data |
	data := GODData new.
	
	data origin: 'email'.
	data author: (self getField: 'From' from: email).
	data title: (self getField: 'Subject' from: email).
	data timestamp: (self getTimestampFrom: email).
	data content: (self getContentFrom: email).
	
	^data.! !

!MAILReceiver class methodsFor: 'accessing' stamp: 'LFSA 11/2/2014 01:04'!
defaultPortNumber
	^110! !

!MAILReceiver class methodsFor: 'converting' stamp: 'LFSA 10/19/2014 22:51'!
getContentFrom: email
	"gets the text/plain part of the email"
	
	| contentDescriptionStart contentStart contentEnd |
	
	contentDescriptionStart := email findString: 'Content-Type: text/plain'.
	(contentDescriptionStart = 0) ifTrue: [^ '']. 
	
	contentStart := email findString: (Character cr) asString, (Character cr) asString startingAt: contentDescriptionStart. "searches for two line breaks in a row"
	(contentStart = 0) ifTrue: [^ '']. 
	
	contentEnd := email findString: '--' startingAt: contentStart.
	(contentEnd = 0) ifTrue: [contentEnd := email size]. 
	
	^ email copyFrom: contentStart + 2 to: contentEnd - 3.  "+2/-3 to remove extra new lines or -"! !

!MAILReceiver class methodsFor: 'converting' stamp: 'gfg 10/19/2014 17:28'!
getField: field from: email
	"gets the field (string) value of the email (a string)"

	| fieldStart fieldEnd fieldLength |
	
	fieldStart := email findString: field, ': '. "searches for the field name followed by a colon and a space"
	(fieldStart = 0) ifTrue: [^ '']. "if the string doesn't have the field, return an empty string"
	
	fieldEnd := email indexOf: Character cr startingAt: fieldStart. "field ends with a line break"
	
	fieldLength := field size.
	
	^ email copyFrom: fieldStart + fieldLength + 2 to: fieldEnd - 1. "+ 2 to skip the colon and the space"! !

!MAILReceiver class methodsFor: 'converting' stamp: 'gfg 10/19/2014 18:22'!
getTimestampFrom: email
	"gets the timestamp from the Date field from the email; needs to remove the weekday so that it is undestood by asTimeStamp"

	| timestampWithWeekday timestampPosition timestampWithoutWeekday|
	timestampWithWeekday := self getField: 'Date' from: email.
	timestampPosition := timestampWithWeekday findString: ', '.
	(timestampPosition = 0) ifTrue: [^ '']. 
	timestampWithoutWeekday := timestampWithWeekday copyFrom: timestampPosition + 2 to: timestampWithWeekday size.
	^ timestampWithoutWeekday asTimeStamp.! !

!MAILReceiver methodsFor: 'initialize-release' stamp: 'LFSA 11/2/2014 00:27'!
initialize
	super initialize.
	cur := 1.
	cache := OrderedCollection new.! !

!MAILReceiver methodsFor: 'initialize-release' stamp: 'LFSA 11/2/2014 00:23'!
initializeConnectionInfo
	super initializeConnectionInfo.
	pop3Client := POP3Client.! !

!MAILReceiver methodsFor: 'accessing' stamp: 'LFSA 10/19/2014 23:40'!
pop3Client: aClient
	"Provides an access point to inject the class used to connect to POP3 server. 
	Used mainly to inject a mock to test this class behavior."
	pop3Client := aClient.! !

!MAILReceiver methodsFor: 'requests' stamp: 'LFSA 11/2/2014 21:17'!
receive
	"Connects to the server, receives new emails and converts them into a list of GodData"
	
	| client messages message data n |
	
	"Validate the connection info: hostname, port, user and password"
	self validateConnectionInfo.
	
	"Initialize the POP3 client"
	client := pop3Client openOnHostNamed: hostname port: port.
	[
		"Log in to POP server"
		client loginUser: user password: password.
				
		"Get the new messages"
		n := client messageCount.
		cur to: n do: [ :messageNr |
			message := client retrieveMessage: messageNr.
			data := MAILReceiver convertToGODData: message.
			cache add: data.
		].
	
		"Update the cursor position"
		cur := n + 1.
		
		"Create a collection with a copy of the cached messages"
		messages := OrderedCollection new.
		messages addAll: cache.
	] ensure: [
		"Close the POP3 connection"
		client close
	].

	^messages.! !

MAIL subclass: #MAILSender
	instanceVariableNames: 'smtpClient'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODEmail'!

!MAILSender class methodsFor: 'converting' stamp: 'LFSA 11/1/2014 20:33'!
convertToHTMLEmail: gData recipient: recipient
	"Convert to MailMessage from GODData"
	
	| message |
	message := MailMessage empty.
	message setField: 'from' toString: gData author.
	message setField: 'to' toString: recipient.
	message setField: 'subject' toString: gData title.
	message setField: 'content-type' toString: 'teste'.

	^message.! !

!MAILSender class methodsFor: 'converting' stamp: 'LFSA 11/1/2014 20:31'!
convertToSimpleEmail: gData recipient: recipient
	"Convert to MailMessage from GODData"
	
	| message|
	
	message := 
'From: ', gData author, '
To: ', recipient, '
Subject: ',  gData title, '
Mime-Version: 1.0
Content-Type: text/plain

', gData content.
	
	^message.! !

!MAILSender class methodsFor: 'accessing' stamp: 'LFSA 11/2/2014 01:04'!
defaultPortNumber
	^25! !

!MAILSender methodsFor: 'initialize-release' stamp: 'LFSA 11/2/2014 20:24'!
initializeConnectionInfo
	super initializeConnectionInfo.
	smtpClient := SMTPClient.! !

!MAILSender methodsFor: 'requests' stamp: 'LFSA 11/2/2014 21:19'!
send: data to: recipient
	"Sends the GODData contained in data to recipient (a string with an email address)"
	
	|recipients message client|
	
	"Validate the connection info: hostname, port, user and password"
	self validateConnectionInfo.
	
	"Check if the required values are set"
	data content isEmpty
		ifTrue: [^Error signal: 'GODData content is empty'].
	
	"Create the collection of recipients"
	recipients := OrderedCollection new.
	recipients add: recipient.
	
	"Convert the GODData to email message"
	message := MAILSender convertToSimpleEmail: data recipient: recipient.
	
	"Initialize the SMTP client"
	client := smtpClient openOnHostNamed: hostname port: port.
	[
		"Lo in to SMTP server"
		client user: user.
		client password: password.
		client login.
		client initiateSession.
		
		"Send the email message"
		client mailFrom: 'mailer'
		          to: recipients
		          text: message.
		client quit.
	] ensure: [
		"Close the SMTP server connection"
		client close.
	].

	^self.! !

!MAILSender methodsFor: 'accessing' stamp: 'LFSA 11/2/2014 20:23'!
smtpClient: aClient
	smtpClient := aClient! !
